class LinkedProgressBar extends GameObject {
	GameObject subject;
	float barWidth, barHeight;
	PVector position;
	color borderColor, backgroundColor, valueColor;
	boolean showLabel = false;
	
	protected float currentValue = 0, targetValue, maxValue = 100;
		
	LinkedProgressBar(GameObject subject, float barWidth, float barHeight, PVector position){
		drawOnUI = true;
	
		this.subject = subject;
		this.barWidth = barWidth;
		this.barHeight = barHeight;
		this.position = position;
		
		borderColor = color(0);
		backgroundColor = color(255);
		valueColor = color(255, 20, 20);
		
		readSubjectValues();
	}
	
	void readSubjectValues(){
		//read the current value of the to-be-displayed property here
	}
	
	void update(float dt){
		readSubjectValues();
		currentValue = targetValue; //Could also use some sort of easing algorithm for prettier animation
	}
	
	void drawUI(){
		//Display the progress bar
		if ( subject != null ){
			pushMatrix();
			  translate(position.x, position.y);
			  drawBar();
			  if ( showLabel ){
				drawLabel();
			  }
			popMatrix();
		}
	}
	
	void drawBar(){
		strokeWeight(1);
		stroke(borderColor);
		fill(backgroundColor);
		rectMode(CORNER);
		rect(0, 0, barWidth, barHeight);

		noStroke();
		fill(getFillColor());
		rect(0, 0, max(barWidth * getPercentage(), 0), barHeight);
	}
	
	void drawLabel(){
		fill(0);
		textAlign(CORNER, CENTER);
		textFont(font);
		textSize(20);
		text(ceil(currentValue), getLabelX(), barHeight/2);
	}
	
	float getPercentage(){
		return constrain((currentValue / maxValue), 0, 1);
	}
	
	float getLabelX(){
		return barWidth + 10;
	}
	
	color getFillColor(){
		return valueColor;
	}
}

class HealthBar extends LinkedProgressBar {
	HealthBar(Ship subject, float barWidth, float barHeight, PVector position){
		super(subject, barWidth, barHeight, position);
	}

	HealthBar(Ship subject){
		this(subject, 200, 12, new PVector(10, height - 22));
	}
	
	void readSubjectValues(){
		targetValue = ((Ship)subject).hitpoints;
		maxValue = ((Ship)subject).maxHitpoints;
	}
}

class PowerBar extends LinkedProgressBar {
	PowerBar(PlayerShip subject, float barWidth, float barHeight, PVector position){
		super(subject, barWidth, barHeight, position);
		valueColor = color(50, 50, 255);
	}

	PowerBar(PlayerShip subject){
		this(subject, 200, 12, new PVector(width - 210, height - 22));
	}
	
	void readSubjectValues(){
		targetValue = ((PlayerShip)subject).power;
		maxValue = ((PlayerShip)subject).maxPower;
	}
	
	color getFillColor(){
		if (targetValue < repulsorCost){
			return lerpColor(color(100), color(150, 150, 150), targetValue / repulsorCost);
		} else {
			return valueColor;
		}
	}
	
	float getLabelX(){
		return -30;
	}
	
	void drawBar(){
		super.drawBar();
		float tickPosition = repulsorCost;
		stroke(0);
		while ( tickPosition < maxValue ){
			final float displayPosition = barWidth * (tickPosition / maxValue);
			line( displayPosition, 0, displayPosition, barHeight );
			tickPosition += repulsorCost;
		}
	}
}