class Mothership extends Bot {
  Vec2 desiredPosition;
  float spawnPeriod = 10;
  float remainingSpawnCooldown = 5;
  
  Vec2 nextKamikazePosition;
  float nextKamikazeRadius;
  float minSpawnGap = 1.0;
  
  Mothership(){
    super();
  }
  
  Mothership(Vec2 startPosition, float radius, float maxHitpoints){
    super(startPosition, radius, maxHitpoints);
    desiredPosition = this.startPosition;
    density = enemyShipDensity * 3;
    maxThrust = 900000;
    maxSpeed = 30;
    
    baseColor = color(255, 179, 0);
	points = 100;
    
    spawnPeriod = random(8, 10);
    remainingSpawnCooldown = spawnPeriod + random(0, 5);
    
    //Pick the parameters for the first kamikaze to be spawned
    nextKamikazeRadius = random(1.0, 2.0);
    nextKamikazePosition = pickKamikazePosition(minSpawnGap, nextKamikazeRadius);
  }  
  
  Mothership(Vec2 startPosition, float radius){
    this(startPosition, radius, 500);
  }
  
  void update(float dt){
    if ( !(dead || isTractored) ){
		//Spawn kamikazes every X seconds
		remainingSpawnCooldown -= dt;
		if ( remainingSpawnCooldown <= 0 ){
		  nextKamikazePosition.addLocal(getPosition());
		  spawnKamikaze(nextKamikazePosition, nextKamikazeRadius);
		  
		  //Choose the parameters for the next one
		  nextKamikazeRadius = random(1.0, 2.0);
		  nextKamikazePosition.set(pickKamikazePosition(minSpawnGap, nextKamikazeRadius));
		  
		  remainingSpawnCooldown = spawnPeriod + remainingSpawnCooldown;
		}
    }
	
    super.update(dt);
  }
  
  Vec2 calculateSteeringVector(){
	//Stay close to the original position
	Vec2 steering = arrive(desiredPosition, 20);
	//Avoid collisions with other Motherships
	steering.addLocal( separate(3, entities.getAll(), Mothership.class).mul(2) );	
	
	return steering;
  }
  
  void draw(){
    if ( !dead ){
      drawNextKamikaze();
    }
    super.draw();
  }
  
  void drawHull(){
    if ( !dead ){
      stroke(0);
      fill(baseColor);
    } else {
      stroke(180, getDeathlyAlpha());
      noFill();
    }
    
    ellipse(0, 0, radius*2, radius*2);
    rectMode(CENTER);
    rect(0, -radius/2, 1.0, radius);
    fill(255, 50, 50);
    
    //Display hitpoints as a growing bar
    if ( !dead ){
      noStroke();
      rect(0, -radius/2, 0.8, map(hitpoints, 0, maxHitpoints, radius, 0) );
    }
  }
  
  void drawNextKamikaze(){
    pushMatrix();
      Vec2 position = body.getPosition();
      translate(position.x, position.y);
      
      float phase = constrain(remainingSpawnCooldown / spawnPeriod, 0, 1);
      stroke(255, 50, 50, map(phase, 0, 1, 255, 0));
      fill(255, 50, 50, map(phase, 0, 1, 255, 0));
      
      line(0, 0, nextKamikazePosition.x, nextKamikazePosition.y);

      float growingDiameter = nextKamikazeRadius * (1 - phase) * 2;
      ellipse(nextKamikazePosition.x, nextKamikazePosition.y, growingDiameter, growingDiameter);
    popMatrix();
  }
  
  void spawnKamikaze(Vec2 kPosition, float kRadius){
    Kamikaze bot = new Kamikaze(
      kPosition,
      kRadius
    );
    entities.add(bot);
  }
  
  Vec2 pickKamikazePosition(float minGap, float kamikazeRadius){
    //Pick a random spot somewhere just outside the mothership's radius
    Vec2 kamikazePosition = new Vec2(0, radius + minGap + kamikazeRadius);
    kamikazePosition = rotateVector(kamikazePosition, radians(random(0, 360)));
    return kamikazePosition;
  }
  
  void maybeSpawnPickup(){
	Vec2 powerupOffset = new Vec2(random(-radius*0.8, radius*0.8), random(-radius*0.8, radius*0.8));
	Pickup powerup = new HealthPickup( getPosition().addLocal(powerupOffset) );
	entities.add(powerup);
  }
}
